@@ -8,7 +8,6 @@
 #define T 64
 
 #define desynchronize() {pr int t; for(t=0;t<$%8;t++);for(t=0;t<$/128;t++);}
-#define wait(myT) {pr int t; for (t=0;t<myT;t++);}
 
 #define SHEAPSIZE 13000
 #define BLOCKSIZE 1
@@ -18,6 +17,7 @@ sh char HEAP[SHEAPSIZE];
 sh int offset = 0;
 sh char *avail[AVAILSIZE];
 sh int high = 0, low = 0;
+sh int buscounter = 0;
 
 
 sync char *newblock( void ) 
@@ -113,28 +113,53 @@ sync char *balloc( pr char *ptr, pr int mode )
   return NULL;
 }
 
+
+int wait( int myT ) 
+{
+  int t;
+  for (t=0;t<myT;t++)
+     ;
+  return 1;
+}
+
+
 sh int USE_JOIN;
 
 void main( void ) 
 {
-  pr int i, t;
-  pr char *p;           /* the value of p is unimportant here. */
-  pr int mode;
+  int i, t;
+  int time;
+  char *p;           /* the value of p is unimportant here. */
+  int mode;
   _ticket[0] = 0;
   _gone[0] = 0;
   if ($==0) { prS("Do you wish to use JOIN? (1/0) "); scanf("%d", &USE_JOIN); }
   barrier;
   srand(4*$*$*$+8*$+17);
+  time = 0;
   desynchronize();
   for (i=0; i<N; i++) {
     wait(rand() % T);   /* wait for a random time between 1 and 6T cycles */
     mode = rand() % 2;  /* toss a coin to determine query mode */
     if (USE_JOIN) {
-       join ( 0, wait(2), 0, 100, 1, p = balloc(p,mode), );
+       join ( 100; wait(2); 1 ) {
+          p = balloc( p, mode );
+          seq syncadd( &buscounter, 1 );  // count bus tours
+       } 
+       else
+          continue;
     }
-    else {
-       if (mode==ALLOC) p=seqballoc(); else seqbfree(p);
+    else {  // asynchronous processing:
+       if (mode==ALLOC)
+            p=seqballoc(); 
+       else seqbfree(p);
     }
   }
   barrier;
+  if ($==0) {
+     time = getct() - time; 
+     printf("number of bus tours: %d\n", buscounter );
+     printf("total time: %d msec\n", time>>8 );
+  }
+  barrier;
 }
