@@ -403,5 +403,30 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
 } \
 /* end of JOIN macro. */
 
-#endif
 
+/*   TRACING ROUTINES:  (see async.c)  990722 */
+
+#define TRACE_END 0
+#define TRACE_SPLIT 1
+#define TRACE_GROUP 2
+#define TRACE_BARRIER_ENTRY 3
+#define TRACE_BARRIER_EXIT 4
+#define TRACE_LOCKUP_WAIT 5
+#define TRACE_LOCKUP_DONE 6
+
+extern sync void initTracing( sh int lengthInWords );
+extern void traceEntry( int evnttype );  // user-defined events
+extern sync void startTracing ( void );
+extern sync void stopTracing ( void );
+extern sync void writeTraceFile( sh char *filename, sh char *title );
+
+extern sh int *_tracebuffer;         // in forklib2.asm
+extern sh int _tracebufferlength;
+extern sh int _tracestarttime;
+extern sh int _tracestoptime;
+extern sh int _traceshsize;
+extern sh int _tracefirstgps;
+extern sh int *_postracebuf;
+extern sh int _tracingenabled;
+
+#endif
