@@ -35,7 +35,7 @@ typedef struct {   /* FILE pointer */
   simple_lock lock; /* to guarantee sequential access to a file */
 } FILE; 
 
-#define __UNBUFALLOC__ malloc(100)   /* buffer for ungetc() */
+#define __UNBUFALLOC__ shmalloc(100)   /* buffer for ungetc() */
 #define __UNBUFSIZE__ 100
 
 extern unsigned int strlen( const char * );
@@ -140,7 +140,8 @@ extern int unmygetchar( int c, char *stdin_buffer );
 
 #define	_NFILE	20	/* initial number of streams: SPARC ABI and default */
 extern int _filepos[_NFILE]; /* store current (physical) r/w position in file */
-#define FILEALLOC malloc   /* allocation routine for generating FILE ptrs */
+#define FILEALLOC shmalloc   /* allocation routine for FILE objects */
+#define FILEFREE shfree      /* deallocation routine for FILE objects */
 
 #define	_IOFBF		0000	/* full buffered */
 #define	_IOLBF		0100	/* line buffered */
