@@ -13,7 +13,7 @@
 
 /* fast integer sqrt()  macro 01/1995 by C.W. Kessler */
 
-extern double sqrt( float );    /* see forklib2.asm */
+extern float sqrt( float );    /* see forklib2.asm */
 
 #define isqrt(p) (1<<((ilog2(p))>>1))
 
@@ -36,6 +36,11 @@ extern float cos( float );
 extern float asin( float ); /* arcus sinus */
 extern float acos( float ); /* arcus cosinus */
 extern float atan( float ); /* arcus tangens */
+extern float acot( float ); /* arcus cotangens */
+extern float arsinh( float ); /* arcus sinus hyperbolicus */
+extern float arcosh( float ); /* arcus cosinus hyperbolicus */
+extern float artanh( float ); /* arcus tangens hyperbolicus */
+extern float arcoth( float ); /* arcus cotangens hyperbolicus */
 
 extern float log10( float );   /* base 10 logarithm */
 
@@ -52,7 +57,8 @@ extern float  ceil( float  );
 extern float  floor( float  );
 extern float  fabs( float  );
 
-extern int abs( int );
+//extern int abs( int );  //stdlib.h
 
 extern float pow( float, float );
+extern float exp( float );
 #endif
