@@ -5,17 +5,20 @@ sh simple_lock screen = 0;
 
 void main(void)
 {
-  join( 100; __NUM_PR__>=1; __RANK__<2)
+ start {
+  initTracing( 10000 );
+  startTracing();
+  farm {
+   join( 100; __NUM_PR__>=1; __RANK__<2)
      farm pprintf("JOIN-BODY! $=%d\n",$);
-  else {
+   else {
      pprintf("ELSE-PART!\n"); 
      continue;  /* go back to join head */
                 /* break would leave it */
+   }
   }
-  barrier;    
-  simple_lockup(&screen);
-    printAccStat();
-  simple_unlock(&screen);
-  barrier;    
+  stopTracing();
+  writeTraceFile("JOINTEST", "test of join statement" );
+ }
 }
 
