@@ -57,6 +57,16 @@ void prI( int number, int leading ) {
 
 static char __hexdigit[] = "0123456789abcdef";
 
+void itohexs( int n, char *buf, int width )
+{
+  int i, z;
+  for ( i=width-1; i>=0; i--) {
+     z = n & 15;
+     buf[i] = __hexdigit[z];
+     n = n>>4;
+  }
+}
+
 void prX( int x ) {
  int e;
  int pos;
@@ -98,10 +108,9 @@ void printAccStat( void )         /*990119*/
 {
   extern int _barrWaitTime;  /*990504*/
 
-  write(1,"AccStat for P", 13); prI(__PROC_NR__,0); write(1,": passed ",9);
-  prI(_barrcnt,0); write(1," barriers, waited ", 18);
-  prI(_barrWaitTime>>8,0);
-  write(1," msecs,\n                ",25);
+  write(1,"AccStat for P", 13);
+  prI(__PROC_NR__,0); 
+  write(1,": ",2);
   prI(_myshldgcnt,0); write(1," of ",4);
   prI(_shldgcnt,0); write(1," sh-loads,\t",11);
   prI(_myshstgcnt,0); write(1," of ",4);
@@ -702,7 +711,7 @@ int fclose ( FILE *fp )
 {
  /* returns zero if closing was OK. */
  int errcode;
- assert( fp );
+ if (! fp ) return 1;
  errcode = close (fp->fd);
  if (!errcode) {
     shfree( fp->buffer );
