@@ -8,9 +8,7 @@
 #include <assert.h>
 #include <io.h>
 #include <stdlib.h>
-
-sh int N;   /* number of array elements */
-sh int *a;  /* the array to be sorted */
+#include <string.h>
 
 #define THRESHOLD 1
 
@@ -29,10 +27,22 @@ void print_array( int *a, int n )
 }
 
 
-/* sequential sorting, using bubble sort.
+int compare( void *a, void *b) { 
+ if (*(int*)a < *(int*)b) return -1;
+ if (*(int*)a > *(int*)b) return 1;
+ return 0;
+}
+
+void seq_sort( int *data, int n, int *presult)
+{
+  memcpy( presult, data, n );
+  qsort( presult, n, sizeof(int), compare );
+}
+
+/* Alternative: sequential sorting, using bubble sort.
  * Assumes that outarray is allocated.
  */
-void seq_sort( int *array, int n, int *outarray )
+void seqsort( int *array, int n, int *outarray )
 {
  int i,j;
  int temp;
@@ -55,7 +65,7 @@ int get_rank( int key, int *array, int n )
  int left = 0;
  int right = n-1;
  int mid;
- if (key >= array[n-1]) return n;
+ if (key > array[n-1]) return n;
  if (key == array[n-1]) return n-1;
  if (key <= array[0]) return 0;
  while (left < right-1) {   /*binary search*/
@@ -71,32 +81,34 @@ int get_rank( int key, int *array, int n )
 
 /* merge array src1 of size n1 and src2 of size n2
  * into one array dest of size n1+n2. 
- * Assertions: p>1, n1*n2>=1, dest array is allocated.
+ * Assertions: p>1, n1*n2>=1, dest + temp array is allocated.
  */
-sync void merge( int *src1, int n1, int *src2, int n2, int *dest)
+sync void merge( sh int *src1, sh int n1, 
+                 sh int *src2, sh int n2,
+                 sh int *dest, sh int *temp)
 {
- sh int p = groupsize();
+ sh int p = 0;
  sh int iter;
- sh int *rank12, *rank21;  /*temp. rank arrays*/
- pr int i;
+ sh int *rank12 = temp,
+        *rank21 = temp + n1;  /*temp. rank arrays*/
+ int i;
+ $ = mpadd( &p, 1 );    // renumber $
  farm assert(p>1);
- rank12 = (int *)shalloc( n1 );
- rank21 = (int *)shalloc( n2 );
  farm pprintf(" merge( src1=%p, n1=%d, src2=%p, n2=%d, dest=%p, n=%d, p=%d)\n",
            src1,n1,src2,n2,dest,n1+n2,p);
  iter = 0;
  farm
    /* self-scheduling par. loop over rank computations: */
-   for (i=mpadd(&iter,1); i<n1; i=mpadd(&iter,1)) 
+   FORALL( i, &iter, 0, n1, 1 )
      rank12[i] = get_rank( src1[i], src2, n2 ); 
  iter = 0;
  farm
-   for (i=mpadd(&iter,1); i<n2; i=mpadd(&iter,1)) 
+   FORALL( i, &iter, 0, n2, 1 )
      rank21[i] = get_rank( src2[i], src1, n1 ); 
  farm {
    /* copy elements to dest using the rank information */
-   for (i=$; i<n1; i+=p)  dest[i+rank12[i]] = src1[i];
-   for (i=$; i<n2; i+=p)  dest[i+rank21[i]] = src2[i];
+   forall (i,0,n1,p)  dest[i+rank12[i]] = src1[i];
+   forall (i,0,n2,p)  dest[i+rank21[i]] = src2[i];
  }
 }
 
@@ -104,46 +116,55 @@ sync void merge( int *src1, int n1, int *src2, int n2, int *dest)
 /* mergesort for an array of size n.
  * The sorted array is to be stored in
  * sortedarray which is assumed to be allocated.
+ * temp is an allocated temporary buffer of length n.
  */
-sync void mergesort( int *array, int n, int *sortedarray )
+sync void mergesort( sh int *array, sh int n, sh int *sortedarray, sh int *temp )
 {
  sh int p = groupsize();
 
  seq pprintf(" mergesort( array=%p, n=%d, p=%d)\n", array,n,p);
 
- if (n<=THRESHOLD || p==1)
-     /*ignore warning: n and p are equal for all callers of this group */
+ if (n<=THRESHOLD) {
      seq seq_sort( array, n, sortedarray );
- else {
-    sh int *temp = (int *)shalloc( n );
-    fork (2; @=$%2; $=$/2)
-       mergesort( array + @*(n/2), (1-@)*(n/2) + @*(n-n/2), temp + @*(n/2) );
-    merge( temp, n/2, temp+n/2, n-n/2, sortedarray );
+     return;
  }
+ if (p==1) {
+     farm seq_sort( array, n, sortedarray );
+     return;
+ }
+ fork (2; @=$%2; $=$/2)
+    mergesort( array + @*(n/2), (1-@)*(n/2) + @*(n-n/2), temp + @*(n/2), 
+               sortedarray + @*(n/2) );
+ merge( temp, n/2, temp+n/2, n-n/2, sortedarray, array );
 }
 
 
 void main( void )
 {
- pr int j;
- if ($==0) {
-    prS("Enter N = ");
-    scanf("%d", &N);
- }
- srand( 8*$*$*$ + 4*$*$ - 18*$ + 37 );
+ int j;
  start {
+   sh int N;   /* number of array elements */
+   sh int *a;  /* the array to be sorted */
    sh int *b;
-   a = (int *) shalloc( N );
-   b = (int *) shalloc( N );
-   farm for (j=$; j<N; j+=__STARTED_PROCS__)  a[j] = rand()%8192; /*set array*/
+   sh int *temp;
+   initTracing(100000);
+   seq {
+     printf("Enter N = ");
+     scanf("%d", &N);
+   }
+   a = (int *) shalloc( N * sizeof(int) );
+   b = (int *) shalloc( N * sizeof(int) );
+   temp = (int *) shalloc( N * sizeof(int) );
+   seq {
+      for (j=0; j<N; j++)
+        a[j] = rand()%8192; /*set array*/
+   }
    seq print_array( a, N );
-   mergesort( a, N, b ); 
+   startTracing();
+   mergesort( a, N, b, temp ); 
+   stopTracing();
    seq print_array( b, N );
+   writeTraceFile( "MERGESORT", "mergesort" );
  }
- barrier; 
- simple_lockup( &screen );
-   printAccStat();
- simple_unlock( &screen );
- barrier; 
  exit(0);
 }
