@@ -80,7 +80,7 @@ extern PSkipListItem PSkipListInsert( PSkipList l, Key key, Inf inf );
 extern PSkipListItem PSkipListDelete( PSkipList l, Key key );
 extern PSkipListItem PSkipListFindMin( PSkipList l );
 extern PSkipListItem PSkipListDeleteMin( PSkipList l );
-extern sync int PSkipListDeleteMins( PSkipList l, int k, PSkipListItem * );
+extern sync int PSkipListDeleteMins( sh PSkipList l, sh int k, sh PSkipListItem * );
 extern PSkipListItem PSkipListChangeInf( PSkipList l, Key key, Inf newInf );
 extern PSkipListItem PSkipListDecreaseKey( PSkipList l, Key oldkey, Key newKey );
 
@@ -542,7 +542,7 @@ PSkipListItem PSkipListDeleteMin( PSkipList l )
  *                              and NULL otherwise
  *  @return int i: the number of found and assigned min items, i<=k.
  */
-sync int PSkipListDeleteMins( PSkipList l, int k, PSkipListItem *ret )
+sync int PSkipListDeleteMins( sh PSkipList l, sh int k, sh PSkipListItem *ret )
 {
   PSkipListItem p;
   int j, m;
