@@ -398,7 +398,7 @@ struct tynode {      /* type nodes: */
 #endif
    long sharedparams;  /* i-tes Bit == 1, falls i-ter Parameter shared  CWK*/
    long regparams;     /* i-tes Bit == 1, falls i-ter Parameter in Reg. CWK*/
-   /*int synchronicity;  /* carrier fuer syncity von Fn-Pointer-Deklarat. CWK*/
+   int synchronicity;  /* carrier fuer syncity von Fn-Pointer-Deklarat. CWK*/
 };
 
 struct field {      /* struct/union fields: */
