@@ -556,6 +556,10 @@ dclproto(Symbol (*dcl),(int,int, char *, Type, Coordinate *, int)); { /*CWK*/
                 }
       } else
          ty1 = dclr(ty, &id, 1);
+      if (ty1 && isfunc(ty1) && (ty1->u.proto)) {
+         (*(ty1->u.proto))->synchronicity = (sy==SYNCF)?1:0;   // 990717
+         //fprint(2,"synchronicity: %d\n", (*(ty1->u.proto))->synchronicity);
+      }
       for (;;) {
          if (Aflag >= 1 && !hasproto(ty1))
             warning("missing prototype\n");
@@ -1085,7 +1089,7 @@ static Type *parameters(lev) {   /*CWK*/
             akt_parreg = 22222;   /*Varargs: keine weiteren Parregs. CWK*/
          }
          else {
-            /*fprint(2,"parameter id = %s\n", id);   /*CWK*/
+            //fprint(2,"parameter id = %s sy = %d\n", id, sy);   /*CWK*/
             p = dclparam(sclass, styp, id, ty, &src, sy); /*CWK*/
             p->parreg = 0;  /*CWK: default*/
             if (styp==SHARED && declaringfunction    /*971124*/
@@ -1215,9 +1219,10 @@ static Type tnode(op, type) Type type; {
    if (type && op==FUNCTION) {   /*CWK*/
       ty->sharedparams = type->sharedparams;
       ty->regparams = type->regparams;
+      ty->synchronicity = type->synchronicity; // 990717
    }
    else 
-        ty->sharedparams = ty->regparams = 0;
+        ty->sharedparams = ty->regparams = ty->synchronicity = 0;
    return ty;
 }
 
