@@ -127,29 +127,38 @@ static Tree call(f, fty, src) Tree f; Type fty; Coordinate src; {
        sharedpar = f->u.sym->sharedparameters;    /*CWK*/
        regpar = f->u.sym->regparameters;    /*CWK*/
        issync = f->u.sym->sync;           /*CWK*/
-       //fprint( 2, "====== %s: sharedpar = %d\n", f->u.sym->name, sharedpar);
+       // fprint( 2, "====== %s: sharedpar = %d issync=%d\n",
+       //         f->u.sym->name, sharedpar, issync);
    }
    else 
      if (isptr(f->type) && isfunc(f->type->type)) {
+       // reorganized 990716
+       // first try to use prototype of pointer variable holding f, if one exists:
        Symbol symb; /*950522 =  f->kids[0]->u.sym;   /*CWK*/
        Tree myf = f->kids[0];  /*CWK 950522*/
        assert(myf);  /*970811*/
        while (myf && !myf->u.sym)  myf = myf->kids[0];   /*950522*/
        symb = myf->u.sym;
-       assert(symb); /*970811*/ /* oben eingebaut in Abfrage 950522*/
        if (symb) {
-#if 0
-         if (f->type->type->u.proto && (*(f->type->type->u.proto))) { // 990611
-            sharedpar = (*(f->type->type->u.proto))->sharedparams;
-            fprint( 2, ">>>>>> %s: sharedpar = %d (was: %d)\n",
-                       symb->name, sharedpar, symb->sharedparameters);
-         }
-         else  // so war's vorher:
-#endif
          sharedpar = symb->sharedparameters;    /*CWK*/
-         //fprint( 2, "###### %s: sharedpar = %d\n", symb->name, sharedpar);
          regpar = symb->regparameters;    /*CWK*/
          issync = symb->sync;
+          // sometimes sharedpar and issync are not properly recognized :-(
+         if (sharedpar && !issync) { // a fn. with shared params must be sync!
+           issync = 1;
+           warning("I converted the call to a synchronous function\n\
+                  via the async pointer %s to a synchronous call!\n", symb->name );
+         }
+         // fprint( 2, "#####+ %s: sharedpar = %d issync=%d\n",
+         //        symb->name, sharedpar, issync);
+       }
+       else // access prototype if existing
+       if (f->type->type &&
+           f->type->type->u.proto && (*(f->type->type->u.proto))) { // 990611
+         sharedpar = (*(f->type->type->u.proto))->sharedparams;
+         issync = (*(f->type->type->u.proto))->synchronicity;
+         // fprint( 2, "###### %s: sharedpar = %d issync=%d\n",
+         //         funcname(f), sharedpar, issync);
        }
        else warning("call to %s without prototype: will treat all parameters as private and function as asynchronous!\n", funcname(f) );  /*CWK*/
      }
@@ -806,9 +815,11 @@ static Tree prefix() {
       Tree ar;     /*Operanden */
       t = gettok();
       expect ('(');
-      if (t != ID)  error("variable expected\n");
+      // if (t != ID)  error("variable expected\n");  // testweise raus 990716
       ar = expr(')');
-      if (ar->type != inttype)  error("integer variable expected\n");
+      if (ar->type != inttype) 
+         //error("integer variable expected\n");  // 990716
+         error("integer expression expected\n");
       p = tree( ILOG2+I, inttype, ar, 0 );
       break;
       }
