@@ -196,10 +196,10 @@ case 'i':
       return INT;
    }
    goto id;
-#if 0
+#if 1
 case 'j':                  /*961024*/
    if (rcp[0] == 'o'
-   &&  rcp[1] == 'j'   /*spaeter i */
+   &&  rcp[1] == 'i'
    &&  rcp[2] == 'n'
    && !(map[rcp[3]]&(DIGIT|LETTER))) {
       cp = rcp + 3;
@@ -426,7 +426,7 @@ case 'w':
    }
    goto id;   /*CWK*/
 case 'h':
-case 'j':
+/*case 'j':*/
 case 'k':
 case 'n':
 case 'o':
