@@ -210,7 +210,6 @@ static void xxxdostmt(lab, swp, lev, rframes) struct swtch *swp; {
    size();
    shframedepth++;
    statement(lab, swp, lev, 0, 0, rframes + 1);
-   shframedepth--;
    definelab(lab + 1);
    expect(WHILE);
    expect('(');
@@ -219,6 +218,7 @@ static void xxxdostmt(lab, swp, lev, rframes) struct swtch *swp; {
    walk(e, lab, 0);
    if (findlabel(lab + 2)->ref)
       definelab(lab + 2);
+   shframedepth--;  // 990712 moved here
    exlgrp();
    synchronize();  /*CWK*/
 }
@@ -504,14 +504,13 @@ static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
       test(')', follow);
    if (glevel == 2) {   /* e1; L: if (!e2) goto L+2; S; L+1: e3; goto L; L+2: */
       if (priv) enlgrp( 1 );    /*CWK*/
+      if (priv) shframedepth++;  // 990712 moved here
       definelab(lab);
       definept(&pt2);
       if (e2)
          walk(e2, 0, lab + 2);
       if (priv && reallypriv) size();    /*CWK, vorher priv */
-      if (priv) shframedepth++;
       statement(lab, swp, lev, 0, 0, rframes + ((priv)? 1:0) ); /*CWK*/
-      if (priv) shframedepth--;
       definelab(lab + 1);
       definept(&pt3);
       if (e3)
@@ -521,6 +520,7 @@ static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
    } else {   /* e1; goto L+3; L: S; L+1: e3; L+3: if (e2) goto L; L+2: */
       int once = 0;
       if (priv) enlgrp( 1 );    /*CWK*/
+      if (priv) shframedepth++;   // 990712 moved here
       if (e2) {
          once = foldcond(e1, e2);
          if (!once)
@@ -546,6 +546,8 @@ static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
    }
    if (findlabel(lab + 2)->ref)
       definelab(lab + 2);
+   if (priv) shframedepth--;   // 990712 nicht vorher, da e2 und e3
+                               // bei sh locals eins tiefer greifen muessen.
    if (priv) exlgrp();    /*CWK*/
    /*if (priv)*/ synchronize();   /*CWK*/
 }
@@ -556,6 +558,7 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    Coordinate pt2, pt3;
    static char follow[] = { IF, ID, '}', 0 };
    extern void mkgrp(), engrp(), exgrp(), synchronize(), ibp(int);  /* in gen.c */
+   extern int shframedepth; 
 
    e1 = e2 = e3 = 0;
    t = gettok();
@@ -594,6 +597,7 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    walk(e2, 0, 0);
    definept(0);
    engrp();
+   shframedepth ++;
    if (kind[t] == ID) {
       e3 = texpr(expr0, ')');
       walk(e3, 0, 0);
@@ -611,6 +615,7 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    /* definelab(lab);  break/continue aus der fork-Anweisung moeglich 17.9. */
    statement(lab, swp, lev, bframes + 1, cframes + 1, rframes + 1);    /*CWK*/
    exgrp();
+   shframedepth --;
    synchronize();    /* nach Ruecksprache mit Helmut Seidl */
 }
 
@@ -975,6 +980,8 @@ void statement(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; {
          error("return from inside a farm statement is forbidden!\n");
       if (!isinsyncmode && isinsideastartbody)  /*970821*/
          warning("return crossing start/join and farm/seq boundary may destroy synchronity\n"); 
+      if (rframes)  /* 990527 count only "harmful" returns */
+         cfunc->nreturns ++;  /* 990525 */
       if (t != ';')
          if (freturn(cfunc->type) == voidtype) {
             error("extraneous return value\n");
@@ -1297,32 +1304,36 @@ static void xxxwhilestmt(lab, swp, lev, rframes) struct swtch *swp; {
    e = texpr(conditional, ')');
    priv = (may_dep_on_pr_var( e )); /*CWK*/
    priv = 1;   /*CWK: wegen break notwendig!*/
-   if (generic(e->op) == CNST || glevel == 2) {   /*CWK: CNST = shared!*/
+   if (generic(e->op) == CNST /* || glevel == 2*/ ) {   /*CWK: CNST = shared!*/
+                              // ^^^^ this makes only trouble. 990712
          /* L, L+1: if (!e) goto L+2; S; goto L; L+2: */
       definelab(lab);
       definelab(lab + 1);
       definept(&pt);
       walk(e, 0, lab + 2);
-      if (priv) shframedepth++;
-      statement(lab, swp, lev, 0, 0, rframes + ((priv)?1:0)); /*CWK*/
-      if (priv) shframedepth--;
+      //if (priv) shframedepth++;   // raus 990712
+      statement(lab, swp, lev, 0, 0, rframes ); // raus + ((priv)?1:0)); /*CWK*/
+      //if (priv) shframedepth--;   // raus 990712
       branch(lab);
+      if (findlabel(lab + 2)->ref)
+         definelab(lab + 2);
    } else {   /* goto L+1; L: S; L+1: if (E) goto L; L+2: */
       if (priv) enlgrp( 0 );    /*CWK*/
+      if (priv) shframedepth++;
       branch(lab + 1);
       definelab(lab);
       if (priv) size();     /*CWK*/
-      if (priv) shframedepth++;
       statement(lab, swp, lev, 0, 0, rframes + ((priv)?1:0)); /*CWK*/
-      if (priv) shframedepth--;
       definelab(lab + 1);
       definept(&pt);
       walk(e, lab, 0);
+      if (priv) shframedepth--;  // 990712 moved across eval(e)
+      if (findlabel(lab + 2)->ref)
+         definelab(lab + 2);
+      if (priv) exlgrp();    /*CWK*/ // 990712 moved upwards:
+                                     // close frame only if opened
    }
-   if (findlabel(lab + 2)->ref)
-      definelab(lab + 2);
    /* keine Nops, weil am Ende synchronisiert wird.  CWK*/
-   if (priv) exlgrp();    /*CWK*/
    synchronize();
    /*CWK, es muss auch bei shared Bedingung immer synchronisiert werden*/
 }
