@@ -648,9 +648,11 @@ static Type tynode(op, type, size, align, ptr) Type type; Generic ptr; {
    if (ptr && op==FUNCTION) {                                     /*CWK*/
       tn->type.sharedparams = (*((Type *)ptr))->sharedparams;     /*CWK*/
       tn->type.regparams = (*((Type *)ptr))->regparams;        /*CWK*/
+      tn->type.synchronicity = (*((Type *)ptr))->synchronicity;  //990716
    } else {
       tn->type.sharedparams = 0;     /*CWK*/
       tn->type.regparams = 0;        /*CWK*/
+      tn->type.synchronicity = 0;        //990716
    }
    BZERO((&tn->type.x), Xtype);
    tn->link = typetable[i];
